<?php

session_start();
if(!isset($_SESSION['domain']) or !preg_match('@^(.*/)core/admin$@', $_SESSION['domain'], $matches) or !isset($_SESSION['profil'])) {
	header('Location: ../../core/admin/');
	exit;
}

# define('PLX_ROOT', preg_replace('@/[^/]+/[^/]+$@', '/', dirname($_SERVER['SCRIPT_FILENAME'])));
define('PLX_ROOT', $matches[1]);

const PLX_CORE = PLX_ROOT .'core/';
include PLX_ROOT . 'config.php';
include PLX_CORE . 'lib/config.php';

define('PATH', PLX_ROOT . PLX_CONFIG_PATH . 'plugins/kzRating/');

# detects accepted language
# $_SERVER['HTTP_ACCEPT_LANGUAGE'] = 'fr-FR,fr;q=0.8,en-US;q=0.5,en;q=0.3'
preg_match_all('@([a-z]{2})(?:-[A-Z]{2})?@', $_SERVER['HTTP_ACCEPT_LANGUAGE'], $m);
foreach(array_unique(array_merge($m[1], array('en', 'fr'))) as $l) {
	$filename = __DIR__ . '/lang/' . $l .'.php';
	if(file_exists($filename)) {
		include $filename;
		break;
	}
}

function i18n($key) {
	global $LANG;
	return isset($LANG[$key]) ? $LANG[$key] : $key;
}

function getFilename($sel) {
	return PATH . $sel . '.xml';
}

function parsefile($sel) {
	$filename = getFilename($sel);
	if(!file_exists($filename)) {
		return false;
	}

	# Mise en place du parseur XML
	$datas = file_get_contents($filename);
	$parser = xml_parser_create(PLX_CHARSET);
	xml_parser_set_option($parser, XML_OPTION_CASE_FOLDING,0);
	xml_parser_set_option($parser, XML_OPTION_SKIP_WHITE,0);
	xml_parse_into_struct($parser, $datas, $values, $iTags);
	xml_parser_free($parser);

	$shieldIps = $values[$iTags['used_ips'][0]]['value'];
	return array(
		'sum'	=> $values[$iTags['total_value'][0]]['value'],
		'ips'	=> explode(PHP_EOL, trim($shieldIps)),
	);
}

function getArticles() {
	$filename = PLX_ROOT . PLX_CONFIG_PATH . 'parametres.xml';

	$datas = file_get_contents($filename);
	$parser = xml_parser_create(PLX_CHARSET);
	xml_parser_set_option($parser, XML_OPTION_CASE_FOLDING, 0);
	xml_parser_set_option($parser, XML_OPTION_SKIP_WHITE, 0);
	xml_parse_into_struct($parser, $datas, $values, $iTags);
	xml_parser_free($parser);

	for($i=0; $i<count($iTags['parametre']); $i++) {
		if($values[ $iTags['parametre'][$i] ]['attributes']['name'] == 'racine_articles') {
			$artsRoot = $values[ $iTags['parametre'][$i] ]['value'];
			break;
		}
	}

	$articles = array();
	foreach(glob(PLX_ROOT . $artsRoot . '*.xml') as $f) {
		if(preg_match('@^((\d{4})\..*)\.xml$@', basename($f), $matches)) {
			$articles[$matches[2]] = $matches[1];
		}
	}
	ksort($articles);
	return array(
		'root' => $artsRoot,
		'articles' => $articles,
	);
}

function getArtTitle($filename) {
	$datas = file_get_contents(PLX_ROOT . $filename);
	$parser = xml_parser_create(PLX_CHARSET);
	xml_parser_set_option($parser, XML_OPTION_CASE_FOLDING,0);
	xml_parser_set_option($parser, XML_OPTION_SKIP_WHITE,0);
	xml_parse_into_struct($parser, $datas, $values, $iTags);
	xml_parser_free($parser);

	return $values[$iTags['title'][0]]['value'];
}

# Localisation des adresses IP pour les votes d'un article et d'une langue
$ref = filter_input(INPUT_GET, 'ref', FILTER_VALIDATE_REGEXP, array('options' => array('regexp' => '@^\d{4}-[a-z]{2,3}$@')));
if(!empty($ref)) {
	include './location.php';

	$datas = parsefile($ref);
	$ips = array_map(
		function($item) {
			return long2ip(base_convert($item, 32, 10));
		},
		$datas['ips']
	);
	header('Content-Type: application/json');
	if(function_exists('getLocations')) {
		# On vérifie que le plugin pour la base de données de géolocalisation est installé
		echo json_encode(is_dir(__DIR__ . '/' . $_SESSION['kzRating-dbPlugin']) ? getLocations($ips) : $ips);
	} else {
		echo json_encode($ips);
	}
	exit;
}

?>
<!DOCTYPE html>
<html lang="fr">
<head>
	<meta http-equiv="X-UA-Compatible" content="IE=Edge" />
	<meta charset="<?= PLX_CHARSET ?>" />
	<meta name="viewport" content="width=device-width, initial-scale=1" />
	<title><?= i18n('STATS') ?></title>
	<link rel="stylesheet" type="text/css" href="../admin.css" />
	<link rel="icon" href="../../core/admin/theme/images/favicon.png" />
</head><body class="kzRating-ip">
	<table>
		<caption><?= i18n('IPS_LOCATION') ?></caption>
		<thead>
			<th><?= i18n('LANGUAGE') ?></th>
			<th><?= i18n('ARTICLE') ?> #</th>
			<th><?= i18n('DATE') ?></th>
			<th><?= i18n('TITLE') ?></th>
			<th><?= i18n('AVERAGE') ?></th>
			<th><?= i18n('BALLOTS') ?></th>
		</thead>
		<tbody id="stats">
<?php

$db = array();
$files = glob(PATH . '*.xml');
$artsDb = getArticles();
$flags = array(
	'de'	=> '🇩🇪',
	'en'	=> '🇬🇧',
	'es'	=> '🇪🇸',
	'fr'	=> '🇫🇷',
	'it'	=> '🇮🇹',
	'nl'	=> '🇳🇱',
	'pl'	=> '🇵🇱',
	'pt'	=> '🇵🇹',
	'ro'	=> '🇷🇴',
	'ru'	=> '🇷🇺',
);

foreach($files as $f) {
	if(preg_match('@^(\d{4}-[a-z]{2,3})\.xml$@', basename($f), $matches)) {
		list($artId, $lang) = explode('-', $matches[1]);
		if(!array_key_exists($artId, $artsDb['articles'])) {
			continue;
		}

		preg_match('@\.(\d{4})(\d{2})(\d{2})(\d{2})(\d{2})\.(.*)$@', $artsDb['articles'][$artId], $dt);
		$datas = parsefile($matches[1]);
		array_walk($datas['ips'], function(&$item, $i) {
			$item = long2ip(base_convert($item, 32, 10));
		});
		$db[$matches[1]] = $datas['ips'];
		$n = count($datas['ips']);
		$artTitle = getArtTitle($artsDb['root'] . $artsDb['articles'][$artId] . '.xml');

		$ip2locationLogo = 'ip2location.png';
?>
			<tr data-ref="<?= $matches[1] ?>">
				<td><?= isset($flags[$lang]) ? '<span title="<?= $lang ?>">' . $flags[$lang] . '</span>' : $lang ?></td>
				<td><?= $artId ?></td>
				<td><?= $dt[1] ?>-<?= $dt[2] ?>-<?= $dt[3] ?> <?= $dt[4] ?>:<?= $dt[5] ?></td>
				<td class="title">
					<a href="../../index.php?article<?= ltrim($artId, '0') ?>/<?= $dt[6] ?>" target="_blank"><?= $artTitle ?></a>
					<a href="../../core/admin/article.php?a=<?= $artId ?>" target="_blank">
						<svg xmlns="http://www.w3.org/2000/svg" width="1rem" viewBox="0 0 576 512">
							<use xlink:href="#edit" />
						</svg>
					</a>
				</td>
				<td class="rating right"><?php if($n > 0) { printf('%.1f%%', $datas['sum'] * 100 / $n); } else { echo '&nbsp;'; } ?></td>
				<td class="right"><span class="ballots"><?= $n ?></span></td>
			</tr>
<?php
	}
}
?>
		</tbody>
	</table>
<?php /* Pour afficher les adresses IP des votants d'un article */ ?>
	<div id="kzRating-ip-container">
		<div>
			<header>
				<h3><?= i18n('ARTICLE') ?> #<span id="kzRating-artId"></span></h3>
				<a href="https://lite.ip2location.com/" target="_blank" title="ip2location"><img src="<?= $ip2locationLogo ?>" <?= getimagesize($ip2locationLogo)[3] ?> alt="logo"></a>
			</header>
			<ul id="kzRating-ip">
			</ul>
		</div>
		<button class="close">❌</button>
	</div>
<?php /* icône pour éditer */ ?>
	<svg xmlns="http://www.w3.org/2000/svg" width="0" viewBox="0 0 576 512">
		<defs>
			<path id="edit" fill="currentColor" d="M402.3 344.9l32-32c5-5 13.7-1.5 13.7 5.7V464c0 26.5-21.5 48-48 48H48c-26.5 0-48-21.5-48-48V112c0-26.5 21.5-48 48-48h273.5c7.1 0 10.7 8.6 5.7 13.7l-32 32c-1.5 1.5-3.5 2.3-5.7 2.3H48v352h352V350.5c0-2.1.8-4.1 2.3-5.6zm156.6-201.8L296.3 405.7l-90.4 10c-26.2 2.9-48.5-19.2-45.6-45.6l10-90.4L432.9 17.1c22.9-22.9 59.9-22.9 82.7 0l43.2 43.2c22.9 22.9 22.9 60 .1 82.8zM460.1 174L402 115.9 216.2 301.8l-7.3 65.3 65.3-7.3L460.1 174zm64.8-79.7l-43.2-43.2c-4.1-4.1-10.8-4.1-14.8 0L436 82l58.1 58.1 30.9-30.9c4-4.2 4-10.8-.1-14.9z" />
		</defs>
	</svg>
	<div class="kzRating-spinner">
		<div></div>
		<div></div>
	</div>
	<script type="text/javascript" src="kzRating-admin.js"></script>
</body></html>
